/* ********************************************************
    dfmax.cpp

    Author = Hideyuki Harada


    Basic Algorithm for resolving MaxClique problem
    "g++ 3.3.3" with "LEDA 4.5"
   ******************************************************** */

using namespace leda;


#include<LEDA/ugraph.h>
#include<LEDA/array.h>
#include<time.h>


#define MAXVERTEX  1000


int Q[MAXVERTEX];  // editting clique vertexes
int Q_count = 0;   // editting clique size

int Qmax[MAXVERTEX];  // MaxClique vertexes
int Qmax_count = 0;   // MaxClique size

int R[MAXVERTEX][MAXVERTEX];  // R[a][b] = depth a, bth node

int Branch_count = 0;  // number of visitted node of state-space tree

ugraph G;

// basic procedure for MaxCliqueProblem
void dfmax(graph G, int [MAXVERTEX][MAXVERTEX], int [MAXVERTEX], int );



// ************************************************************



int main(void){

  int i;
  float T = used_time();
  int num_vertex;  // size of R
  node v;

  G.read("graph.gw");
  G.make_undirected();

  num_vertex = 0;

  // format R0 = V
  forall_nodes(v, G){
    R[0][++num_vertex] = index(v);
  }


  dfmax(G, R, Q, num_vertex);


  // show result
  
  cout << endl;
  cout << "run time = " << used_time(T) << endl;
  cout << "Branch count = " << Branch_count << endl;
  cout << "MAX CLIQUE SIZE = " << Qmax_count << endl;
  cout << "CLIQUE ELEMENT " << endl;

  cout << Qmax[0];
  for( i = 1; i < Qmax_count; i++ ){
    cout << " " << Qmax[i];
  }

  cout << endl;

  return 0;
}


// *****************************************************************

void dfmax(graph G, int R[][MAXVERTEX+1], int Q[], int num_vertex){

  int  i, j, p, q;
  int  deg;

  node v;

  list<node> L;

  int  neighbor[MAXVERTEX+1];
  int  Rp_count;

  while(num_vertex != 0){

    Branch_count++;

    for(i = 0; i < num_vertex; i++){
      neighbor[i] = 0;
      R[Q_count+1][i] = 0;
    }

    // select p from R
    p = R[Q_count][0];  
    // add p to Q
    Q[++Q_count] = p;


    deg = 0;

    // Rp = intersect R & neighbor(p)
    forall_adj_nodes(v, p){
      neighbor[++deg] = index(v);
    }

    Rp_count = 0;

    for(i = 0; i < num_vertex; i++){
      for(j = 0; j < deg; j++){
        if( R[q][i] == neighbor[j] ){
          R[Q_count][++Rp_count] = neighbor[j];
        }
      }
    }


    // recursive procedure
    if( Rp_count != 0 ){
      dfmax(G, R, Q, Rp_count);
    }


    // update MaxClique
    else if(Q_count > Qmax_count){
      Qmax_count = Q_count;
      for(i = 0; i < Q_count; i++){
        Qmax[i] = Q[i];
      }
    }


    // remove p from Q
    Q[Q_count--] = 0;    
    // remove p from R
    R[Q_count][num_vertex--] = 0;
  }
}


